
#include "esapitools.h"

#include "gsdlunicode.h"

#include "EsapiCommon.h"
#include "EncoderConstants.h"
#include "reference/DefaultEncoder.h"

#include "codecs/Codec.h"
#include "codecs/UnixCodec.h"
#include "codecs/WindowsCodec.h"
#include "codecs/HTMLEntityCodec.h"

#include "util/TextConvert.h"
#include "crypto/CryptoppCommon.h"

#include "errors/NullPointerException.h"
#include "errors/UnsupportedOperationException.h"


esapi::NarrowString text_t_to_esapi_narrow_string(text_t& in)
{
    text_t in_short_padded_utf8 = to_utf8(in.begin(),in.end());
    char* in_utf8 = in_short_padded_utf8.getcstr(); // MSB of each array position in the text_t will be 0
    esapi::NarrowString out_nstring(in_utf8); // NarrowString is typedef for std::string

    delete [] in_utf8;

    return out_nstring;
}



esapi::WideString text_t_to_esapi_wide_string(text_t& in)
{
    text_t in_short_padded_utf8 = to_utf8(in.begin(),in.end());
    char* in_utf8 = in_short_padded_utf8.getcstr(); // MSB of each array position in the text_t will be 0
    esapi::NarrowString* in_nstring = new esapi::NarrowString(in_utf8); // NarrowString is typedef for std::string

    esapi::WideString out_wstring = TextConvert::NarrowToWide(*in_nstring); // esapi::WideString is typedef for std::wstring

    delete [] in_utf8;
    delete in_nstring;

    return out_wstring;

}


text_t esapi_narrow_string_to_text_t(esapi::NarrowString& in)
{
    const char* in_cstr = in.c_str();

    text_t out_text_t(in_cstr);

    return out_text_t;
}

text_t esapi_wide_string_to_text_t(esapi::WideString& in)
{

    esapi::NarrowString in_nstring = TextConvert::WideToNarrow(in); // esapi::NarrowString is typedef for std::string
    const char* in_cstr = in_nstring.c_str();

    text_t out_text_t(in_cstr);

    return out_text_t;
}



static esapi::StringArray Make_HTML_Vector()
  {
    esapi::StringArray sa;
    sa.push_back(","); sa.push_back(".");
    sa.push_back("-"); sa.push_back("_");
    sa.push_back(" ");
    return sa;
  }

  static esapi::StringArray Make_HTMLATTR_Vector()
  {
    esapi::StringArray sa;
    sa.push_back(","); sa.push_back(".");
    sa.push_back("-"); sa.push_back("_");
    return sa;
  }

  static esapi::StringArray Make_CSS_Vector()
  {
    esapi::StringArray sa;
    sa.push_back("");
    return sa;
  }

  static esapi::StringArray Make_JAVASCRIPT_Vector()
  {
    esapi::StringArray sa;
    sa.push_back(","); sa.push_back(".");
    sa.push_back("_");
    return sa;
  }

  static esapi::StringArray Make_VBSCRIPT_Vector()
  {
    esapi::StringArray sa;
    sa.push_back(","); sa.push_back(".");
    sa.push_back("_");
    return sa;
  }

  static esapi::StringArray Make_XML_Vector()
  {
    esapi::StringArray sa;
    sa.push_back(","); sa.push_back(".");
    sa.push_back("-"); sa.push_back("_");
    sa.push_back(" ");
    return sa;
  }

  static esapi::StringArray Make_SQL_Vector()
  {
    esapi::StringArray sa;
    sa.push_back(" ");
    return sa;
  }

  static esapi::StringArray Make_OS_Vector()
  {
    esapi::StringArray sa;
    sa.push_back("-");
    return sa;
  }

  static esapi::StringArray Make_XMLATTR_Vector()
  {
    esapi::StringArray sa;
    sa.push_back(","); sa.push_back(".");
    sa.push_back("-"); sa.push_back("_");
    return sa;
  }

  static esapi::StringArray Make_XPATH_Vector()
  {
    esapi::StringArray sa;
    sa.push_back(","); sa.push_back(".");
    sa.push_back("-"); sa.push_back("_");
    sa.push_back(" ");
    return sa;
  }

static const esapi::StringArray IMMUNE_HTML = Make_HTML_Vector();
static const esapi::StringArray IMMUNE_HTMLATTR = Make_HTMLATTR_Vector();
static const esapi::StringArray IMMUNE_CSS = Make_CSS_Vector();
static const esapi::StringArray IMMUNE_JAVASCRIPT = Make_JAVASCRIPT_Vector();
static const esapi::StringArray IMMUNE_VBSCRIPT = Make_VBSCRIPT_Vector();
static const esapi::StringArray IMMUNE_XML = Make_XML_Vector();
static const esapi::StringArray IMMUNE_SQL = Make_SQL_Vector();
static const esapi::StringArray IMMUNE_OS = Make_OS_Vector();
static const esapi::StringArray IMMUNE_XMLATTR = Make_XMLATTR_Vector();
static const esapi::StringArray IMMUNE_XPATH = Make_XPATH_Vector();

//static esapi::LDAPCodec ldapCodec;
static esapi::HTMLEntityCodec htmlCodec;
/*static esapi::XMLEntityCodec xmlCodec;
static esapi::PercentCodec percentCodec;
static esapi::JavaScriptCodec javaScriptCodec;
static esapi::VBScriptCodec vbScriptCodec;
static esapi::CSSCodec cssCodec;
*/
esapi::NarrowString encodeForHTML(const esapi::NarrowString & input) {
  return htmlCodec.encode( IMMUNE_HTML, input);  
}
